# -*- coding: utf-8 -*-

# ==============================================================================
# AUTOGENERATED
# python.sdk.version - AUTOGENERATED
# VERSION.py - MAINTAINER's. Don't edit, if you don't know what are you doing
# ==============================================================================

VERSION = (0, 0, 7, 0)
RELEASE_SUFFIX = ''

VERSION_STRING = '.'.join([str(x) for x in VERSION])
RELEASE_STRING = "v{}{}".format(VERSION_STRING, RELEASE_SUFFIX)

PROJECT = 'Route4Me Python SDK'
COPYRIGHT = '2016-2020 © Route4Me Python Team'
AUTHOR = 'Route4Me Python Team (SDK)'
AUTHOR_EMAIL = 'juan@route4me.com'
TITLE = 'route4me'
LICENSE = 'ISC'
BUILD = None   # TRAVIS_COMMIT
COMMIT = None  # TRAVIS_BUILD_NUMBER
