# -*- coding: utf-8 -*-
# codebeat:disable[TOO_MANY_FUNCTIONS, LOC, ABC, ARITY, TOTAL_LOC]

MAIN_HOST = 'https://api.route4me.com'

ACTIVITY_FEED = '{0}/api.v4/activity_feed.php'.format(MAIN_HOST)
ADDRESSBOOK = '{0}/api.v4/address_book.php'.format(MAIN_HOST)
ADDRESS_HOST = '{}/api.v4/address.php'.format(MAIN_HOST)
ADD_ROUTE_NOTES_HOST = '{0}/actions/addRouteNotes.php'.format(MAIN_HOST)
API_HOST = '{0}/api.v4/optimization_problem.php'.format(MAIN_HOST)
MEMBER_AUTHENTICATE = '{0}/actions/authenticate.php'.format(MAIN_HOST)
AVOIDANCE = '{0}/api.v4/avoidance.php'.format(MAIN_HOST)
BATCH_GEOCODER = '{0}/api/geocoder.php'.format(MAIN_HOST)
VERIFY_DEVICE_LICENSE = '{0}/api/device/verify_device_license.php'.format(MAIN_HOST)
DEVICE_LOCATION_URL = '{0}/api/track/get_device_location.php'.format(MAIN_HOST)
DUPLICATE_ROUTE = '{0}/actions/duplicate_route.php'.format(MAIN_HOST)
EXPORTER = '{0}/actions/route/export_current_route.php'.format(MAIN_HOST)
FILE_UPLOAD_HOST = '{0}/actions/upload/upload.php'.format(MAIN_HOST)
FILE_UPLOAD_PREVIEW_HOST = '{0}/actions/upload/csv-xls-preview.php'.format(MAIN_HOST)
FILE_UPLOAD_GEOCODE_HOST = '{0}/actions/upload/csv-xls-geocode.php'.format(MAIN_HOST)
GET_ACTIVITIES_HOST = '{0}/api/get_activities.php'.format(MAIN_HOST)
GET_USERS_HOST = '{0}/api/member/view_users.php'.format(MAIN_HOST)
MERGE_ROUTES_HOST = '{0}/actions/merge_routes.php'.format(MAIN_HOST)
MOVE_ROUTE_DESTINATION = '{0}/actions/route/move_route_destination.php'.format(MAIN_HOST)
ORDERS_HOST = '{0}/api.v4/order.php'.format(MAIN_HOST)
RAPID_ADDRESS = 'https://rapid.route4me.com/street_data/'
RAPID_ADDRESS_SERVICE = 'https://rapid.route4me.com/street_data/service/'
RAPID_ADDRESS_ZIP = 'https://rapid.route4me.com/street_data/zipcode/'
REGISTER_ACTION = '{0}/actions/register_action.php'.format(MAIN_HOST)
RESEQUENCE_ROUTE = '{0}/api.v3/route/reoptimize_2.php'.format(MAIN_HOST)
ROUTE_HOST = '{0}/api.v4/route.php'.format(MAIN_HOST)
SET_GPS_HOST = '{0}/track/set.php'.format(MAIN_HOST)
SHARE_ROUTE_HOST = '{0}/actions/route/share_route.php'.format(MAIN_HOST)
SHOW_ROUTE_HOST = '{0}/route4me.php'.format(MAIN_HOST)
SINGLE_GEOCODER = '{0}/api/address.php'.format(MAIN_HOST)
TERRITORY_HOST = '{0}/api.v4/territory.php'.format(MAIN_HOST)
USER_URL = '{0}/api.v4/user.php'.format(MAIN_HOST)
USER_LICENSE_HOST = '{0}/api/member/user_license.php'.format(MAIN_HOST)
VALIDATE_SESSION = '{0}/datafeed/session/validate_session.php'.format(MAIN_HOST)
VEHICLES_HOST = '{}/api/vehicles/view_vehicles.php'.format(MAIN_HOST)
WEBINAR_REGISTER = '{0}/actions/webinar_register.php'.format(MAIN_HOST)
ZIPCODE_TERRITORIES = 'https://rapid.route4me.com/street_data/territories'
